@@ -9,7 +9,7 @@
 from __future__ import absolute_import
 
 import sys
-if sys.version_info >= (3,0):
+if sys.version_info >= (3, 0):
     from .Custom_httplib3x import httplib
 else:
     from .Custom_httplib27 import httplib
@@ -23,16 +23,12 @@ except ImportError:
     from urllib.parse import urlparse
 
 from .Config import Config
-from .Exceptions import ParameterError
+from .Exceptions import ParameterError, S3SSLCertificateError
 from .Utils import getBucketFromHostname
 
-if not 'CertificateError' in ssl.__dict__:
-    class CertificateError(Exception):
-        pass
-else:
-    CertificateError = ssl.CertificateError
 
-__all__ = [ "ConnMan" ]
+
+__all__ = ["ConnMan"]
 
 
 class http_connection(object):
@@ -128,11 +124,13 @@ class http_connection(object):
         cert = self.c.sock.getpeercert()
         try:
             ssl.match_hostname(cert, self.hostname)
-        except AttributeError: # old ssl module doesn't have this function
+        except AttributeError:
+            # old ssl module doesn't have this function
             return
-        except ValueError: # empty SSL cert means underlying SSL library didn't validate it, we don't either.
+        except ValueError:
+            # empty SSL cert means underlying SSL library didn't validate it, we don't either.
             return
-        except CertificateError as e:
+        except S3CertificateError as e:
             if not self.forgive_wildcard_cert(cert, self.hostname):
                 raise e
 
@@ -259,16 +257,27 @@ class ConnMan(object):
     @staticmethod
     def put(conn):
         if conn.id.startswith("proxy://"):
-            conn.c.close()
+            ConnMan.close(conn)
             debug("ConnMan.put(): closing proxy connection (keep-alive not yet supported)")
             return
 
         if conn.counter >= ConnMan.conn_max_counter:
-            conn.c.close()
+            ConnMan.close(conn)
             debug("ConnMan.put(): closing over-used connection")
             return
 
+        cfg = Config()
+        if not cfg.connection_pooling:
+            ConnMan.close(conn)
+            debug("ConnMan.put(): closing connection (connection pooling disabled)")
+            return
+
         ConnMan.conn_pool_sem.acquire()
         ConnMan.conn_pool[conn.id].append(conn)
         ConnMan.conn_pool_sem.release()
         debug("ConnMan.put(): connection put back to pool (%s#%d)" % (conn.id, conn.counter))
+
+    @staticmethod
+    def close(conn):
+        if conn:
+            conn.c.close()
