@@ -12,7 +12,6 @@ import sys
 import os
 import time
 import errno
-import base64
 import mimetypes
 import io
 import pprint
@@ -25,6 +24,12 @@ try:
     from urlparse import urlparse
 except ImportError:
     from urllib.parse import urlparse
+try:
+    # Python 2 support
+    from base64 import encodestring
+except ImportError:
+    # Python 3.9.0+ support
+    from base64 import encodebytes as encodestring
 
 import select
 
@@ -44,7 +49,7 @@ from .MultiPart import MultiPartUpload
 from .S3Uri import S3Uri
 from .ConnMan import ConnMan
 from .Crypto import (sign_request_v2, sign_request_v4, checksum_sha256_file,
-                    checksum_sha256_buffer, s3_quote, format_param_str)
+                     checksum_sha256_buffer, s3_quote, format_param_str)
 
 try:
     from ctypes import ArgumentError
@@ -156,11 +161,10 @@ class S3Request(object):
     def use_signature_v2(self):
         if self.s3.endpoint_requires_signature_v4:
             return False
-        # in case of bad DNS name due to bucket name v2 will be used
-        # this way we can still use capital letters in bucket names for the older regions
 
-        if self.resource['bucket'] is None or not check_bucket_name_dns_conformity(self.resource['bucket']) or self.s3.config.signature_v2 or self.s3.fallback_to_signature_v2:
+        if self.s3.config.signature_v2 or self.s3.fallback_to_signature_v2:
             return True
+
         return False
 
     def sign(self):
@@ -272,6 +276,17 @@ class S3(object):
             host = getHostnameFromBucket(bucket)
         else:
             host = self.config.host_base.lower()
+        # The following hack is needed because it looks like that some servers
+        # are not respecting the HTTP spec and so will fail the signature check
+        # if the port is specified in the "Host" header for default ports.
+        # STUPIDIEST THING EVER FOR A SERVER...
+        # See: https://github.com/minio/minio/issues/9169
+        if self.config.use_https:
+            if host.endswith(':443'):
+                host = host[:-4]
+        elif host.endswith(':80'):
+            host = host[:-3]
+
         debug('get_hostname(%s): %s' % (bucket, host))
         return host
 
@@ -286,7 +301,9 @@ class S3(object):
              or (bucket_name not in S3Request.redir_map
                 and not check_bucket_name_dns_support(self.config.host_bucket, bucket_name))
             ):
-                uri = "/%s%s" % (bucket_name, resource['uri'])
+                uri = "/%s%s" % (s3_quote(bucket_name, quote_backslashes=False,
+                                          unicode_output=True),
+                                 resource['uri'])
         else:
             uri = resource['uri']
         if base_path:
@@ -382,8 +399,6 @@ class S3(object):
             bucket_location = bucket_location.strip()
             if bucket_location.upper() == "EU":
                 bucket_location = bucket_location.upper()
-            else:
-                bucket_location = bucket_location.lower()
             body  = "<CreateBucketConfiguration><LocationConstraint>"
             body += bucket_location
             body += "</LocationConstraint></CreateBucketConfiguration>"
@@ -968,7 +983,7 @@ class S3(object):
         request = self.create_request("BUCKET_LIST", bucket = uri.bucket(),
                                       uri_params = {'policy': None})
         response = self.send_request(request)
-        return response['data']
+        return decode_from_s3(response['data'])
 
     def set_policy(self, uri, policy):
         headers = SortedDict(ignore_case = True)
@@ -991,7 +1006,7 @@ class S3(object):
         request = self.create_request("BUCKET_LIST", bucket = uri.bucket(),
                                       uri_params = {'cors': None})
         response = self.send_request(request)
-        return response['data']
+        return decode_from_s3(response['data'])
 
     def set_cors(self, uri, cors):
         headers = SortedDict(ignore_case = True)
@@ -1227,12 +1242,20 @@ class S3(object):
 
         raise S3Error(response)
 
-    def send_request(self, request, retries = _max_retries):
-        if request.resource.get('bucket') \
-           and not request.use_signature_v2() \
-           and S3Request.region_map.get(request.resource['bucket'],
-                                        Config().bucket_location) == "US":
-            debug("===== Send_request inner request to determine the bucket region =====")
+    def update_region_inner_request(self, request):
+        """Get and update region for the request if needed.
+
+        Signature v4 needs the region of the bucket or the request will fail
+        with the indication of the correct region.
+        We are trying to avoid this failure by pre-emptively getting the
+        correct region to use, if not provided by the user.
+        """
+        if request.resource.get('bucket') and not request.use_signature_v2() \
+           and S3Request.region_map.get(
+                request.resource['bucket'], Config().bucket_location
+               ) == "US":
+            debug("===== SEND Inner request to determine the bucket region "
+                  "=====")
             try:
                 s3_uri = S3Uri(u's3://' + request.resource['bucket'])
                 # "force_us_default" should prevent infinite recursivity because
@@ -1240,11 +1263,16 @@ class S3(object):
                 region = self.get_bucket_location(s3_uri, force_us_default=True)
                 if region is not None:
                     S3Request.region_map[request.resource['bucket']] = region
-                debug("===== END send_request inner request to determine the bucket region (%r) =====",
-                      region)
+                debug("===== SUCCESS Inner request to determine the bucket "
+                      "region (%r) =====", region)
             except Exception as exc:
                 # Ignore errors, it is just an optimisation, so nothing critical
-                debug("Error getlocation inner request: %s", exc)
+                debug("getlocation inner request failure reason: %s", exc)
+                debug("===== FAILED Inner request to determine the bucket "
+                      "region =====")
+
+    def send_request(self, request, retries = _max_retries):
+        self.update_region_inner_request(request)
 
         request.body = encode_to_s3(request.body)
         headers = request.headers
@@ -1270,6 +1298,10 @@ class S3(object):
                 attrs = parse_attrs_header(response["headers"]["x-amz-meta-s3cmd-attrs"])
                 response["s3cmd-attrs"] = attrs
             ConnMan.put(conn)
+        except (S3SSLError, S3SSLCertificateError):
+            # In case of failure to validate the certificate for a ssl
+            # connection,no need to retry, abort immediately
+            raise
         except (IOError, Exception) as e:
             debug("Response:\n" + pprint.pformat(response))
             if ((hasattr(e, 'errno') and e.errno
@@ -1281,10 +1313,9 @@ class S3(object):
             # When the connection is broken, BadStatusLine is raised with py2
             # and RemoteDisconnected is raised by py3 with a trap:
             # RemoteDisconnected has an errno field with a None value.
-            if conn:
-                # close the connection and re-establish
-                conn.counter = ConnMan.conn_max_counter
-                ConnMan.put(conn)
+
+            # close the connection and re-establish
+            ConnMan.close(conn)
             if retries:
                 warning("Retrying failed request: %s (%s)" % (resource['uri'], e))
                 warning("Waiting %d sec..." % self._fail_wait(retries))
@@ -1345,23 +1376,7 @@ class S3(object):
     def send_file(self, request, stream, labels, buffer = '', throttle = 0,
                   retries = _max_retries, offset = 0, chunk_size = -1,
                   use_expect_continue = None):
-        if request.resource.get('bucket') \
-           and not request.use_signature_v2() \
-           and S3Request.region_map.get(request.resource['bucket'],
-                                        Config().bucket_location) == "US":
-            debug("===== Send_file inner request to determine the bucket region =====")
-            try:
-                s3_uri = S3Uri(u's3://' + request.resource['bucket'])
-                # "force_us_default" should prevent infinite recursivity because
-                # it will set the region_map dict.
-                region = self.get_bucket_location(s3_uri, force_us_default=True)
-                if region is not None:
-                    S3Request.region_map[request.resource['bucket']] = region
-                debug("===== END Send_file inner request to determine the bucket region (%r) =====",
-                      region)
-            except Exception as exc:
-                # Ignore errors, it is just an optimisation, so nothing critical
-                debug("Error getlocation inner request: %s", exc)
+        self.update_region_inner_request(request)
 
         if use_expect_continue is None:
             use_expect_continue = self.config.use_http_expect
@@ -1614,23 +1629,7 @@ class S3(object):
         return response
 
     def recv_file(self, request, stream, labels, start_position = 0, retries = _max_retries):
-        if request.resource.get('bucket') \
-           and not request.use_signature_v2() \
-           and S3Request.region_map.get(request.resource['bucket'],
-                                        Config().bucket_location) == "US":
-            debug("===== Recv_file inner request to determine the bucket region =====")
-            try:
-                s3_uri = S3Uri(u's3://' + request.resource['bucket'])
-                # "force_us_default" should prevent infinite recursivity because
-                # it will set the region_map dict.
-                region = self.get_bucket_location(s3_uri, force_us_default=True)
-                if region is not None:
-                    S3Request.region_map[request.resource['bucket']] = region
-                debug("===== END recv_file Inner request to determine the bucket region (%r) =====",
-                      region)
-            except Exception as exc:
-                # Ignore errors, it is just an optimisation, so nothing critical
-                debug("Error getlocation inner request: %s", exc)
+        self.update_region_inner_request(request)
 
         method_string, resource, headers = request.get_triplet()
         filename = stream.stream_name
@@ -1662,8 +1661,6 @@ class S3(object):
             debug("Response:\n" + pprint.pformat(response))
         except ParameterError as e:
             raise
-        except OSError as e:
-            raise
         except (IOError, Exception) as e:
             if self.config.progress_meter:
                 progress.done("failed")
@@ -1672,10 +1669,9 @@ class S3(object):
                 or "[Errno 104]" in str(e) or "[Errno 32]" in str(e)
                ) and not isinstance(e, SocketTimeoutException):
                 raise
-            if conn:
-                # close the connection and re-establish
-                conn.counter = ConnMan.conn_max_counter
-                ConnMan.put(conn)
+
+            # close the connection and re-establish
+            ConnMan.close(conn)
 
             if retries:
                 warning("Retrying failed request: %s (%s)" % (resource['uri'], e))
@@ -1769,8 +1765,7 @@ class S3(object):
                ) and not isinstance(e, SocketTimeoutException):
                 raise
             # close the connection and re-establish
-            conn.counter = ConnMan.conn_max_counter
-            ConnMan.put(conn)
+            ConnMan.close(conn)
 
             if retries:
                 warning("Retrying failed request: %s (%s)" % (resource['uri'], e))
@@ -1838,7 +1833,7 @@ def parse_attrs_header(attrs_header):
 
 def compute_content_md5(body):
     m = md5(encode_to_s3(body))
-    base64md5 = base64.encodestring(m.digest())
+    base64md5 = encodestring(m.digest())
     base64md5 = decode_from_s3(base64md5)
     if base64md5[-1] == '\n':
         base64md5 = base64md5[0:-1]
