@@ -64,9 +64,9 @@ def parseNodes(nodes):
     retval = []
     for node in nodes:
         retval_item = {}
-        for child in node.getchildren():
+        for child in node:
             name = decode_from_s3(child.tag)
-            if child.getchildren():
+            if len(child):
                 retval_item[name] = parseNodes([child])
             else:
                 found_text = node.findtext(".//%s" % child.tag)
@@ -101,7 +101,7 @@ def stripNameSpace(xml):
 __all__.append("stripNameSpace")
 
 def getTreeFromXml(xml):
-    xml, xmlns = stripNameSpace(xml)
+    xml, xmlns = stripNameSpace(encode_to_s3(xml))
     try:
         tree = ET.fromstring(xml)
         if xmlns:
@@ -122,8 +122,8 @@ __all__.append("getListFromXml")
 
 def getDictFromTree(tree):
     ret_dict = {}
-    for child in tree.getchildren():
-        if child.getchildren():
+    for child in tree:
+        if len(child):
             ## Complex-type child. Recurse
             content = getDictFromTree(child)
         else:
@@ -194,12 +194,12 @@ __all__.append("dateRFC822toUnix")
 def formatSize(size, human_readable = False, floating_point = False):
     size = floating_point and float(size) or int(size)
     if human_readable:
-        coeffs = ['k', 'M', 'G', 'T']
+        coeffs = ['K', 'M', 'G', 'T']
         coeff = ""
         while size > 2048:
             size /= 1024
             coeff = coeffs.pop(0)
-        return (size, coeff)
+        return (floating_point and float(size) or int(size), coeff)
     else:
         return (size, "")
 __all__.append("formatSize")
